clear
clc

options=optimset('TolCon',1e-13,'TolFun',1e-13,'TolX',1e-13,'Display','off','MaxFunEvals',10000000,'MaxIter',2000);

alternative_tax=0;%Set =1 to calibrate marginal tax rates as in Figure 5 in the appendix. Set to zero to have the case in the main body





%Define Income grid
income_start=2500;
income_end=5002500

n=(income_end-income_start)/2500+1;
income=linspace(income_start,income_end,n)';
y=income;

%elasticitiy of substitution
el_sub=3.1;
rho=1-1./el_sub;

%labor supply elasticity
elasticity=.33;





%Calibrate the income distribution. Start with lognormal part
mu_inc=10;
sigma=.95;
pareto=2.5;
exp(mu_inc+sigma^2/2)



%Now add Pareto tail for incomes >150 000, where the thickness parameter
%decreasses linearly between 150 000 and 350 000

pareto_start=150000
index_pareto_start=find(income>pareto_start,1,'first');


pareto_constant=350000;
index_pareto_constant=find(income>pareto_constant,1,'first');

cutoff=index_pareto_constant;
pareto_paraeter_start=1/((1-logncdf(pareto_start,mu_inc,sigma))./(lognpdf(pareto_start,mu_inc,sigma).*pareto_start))


pareto_vector=linspace(pareto_paraeter_start-1.55,0,index_pareto_constant-index_pareto_start+1);
if pareto_paraeter_start-pareto_paraeter_start>1
    disp('uncorrect smoothing')
    return
end
vector_help=linspace(0,(pareto_paraeter_start-1.55),index_pareto_constant-index_pareto_start+1);

vector = 1./(1+exp(-1 .*(vector_help-(pareto_paraeter_start-1.5)./2)));
vector=vector';
vector2=flipud(vector_help');

pareto_vector=1.5+vector2;

plot(pareto_vector)


pmf=lognpdf(income,mu_inc,sigma);
interesting=zeros(n,1);
for i=index_pareto_start:index_pareto_constant
    if i>index_pareto_start
        pmf(i) = ( pareto*x_min^pareto*income(i)^(-pareto-1));
        
    end
    pareto=pareto_vector(i-index_pareto_start+1);
    x_min = ((pmf(i)*income(i)^(pareto+1))/pareto)^(1/pareto);
    
    pmf(i) = ( pareto*x_min^pareto*income(i)^(-pareto-1));
    interesting(i) = x_min;
    
end
for i=index_pareto_constant:n
    pmf(i) = ( pareto*x_min^pareto*income(i)^(-pareto-1));
end

pmf= ksdensity(income,income,'weights',pmf,'width',20000);

%probability mass function of income
pmf=pmf./sum(pmf);

%cdf of income
cdf=cumsum(pmf);






%Now define welfare weights.

%weights in welfare function. Chose equal to pmf to obtain Utilitarian objective
weights=pmf;
%Chose curvature of utility cardinalization
gamma_crra=1;





%Choice of baseline tax schedule



tau_tax=-3;
p_tax=.15

t_prime = 1 - (1-tau_tax) .* income.^(-p_tax);
t_prime_prime=p_tax*(1-tau_tax).*income.^(-p_tax-1);
Tax = income - (1-tau_tax)/(1-p_tax) .*income.^(1-p_tax);



%Alternatively calibrate marginal tax rates as in Figure 5 in the appendix. 
if alternative_tax==1
    
    
    t_prime_hsv=t_prime;
    b_tax=.317;
    s_tax=0.016;
    p_tax=0.940;
    
    t_prime=b_tax*  (        1-    (s_tax*((income*(1/1000)).^p_tax) +1).^(-1/(p_tax)-1  )    );
    
    
    
    
    
    Tax=income.*b_tax.*(1-(s_tax.*(income/1000).^p_tax+1).^(-1./p_tax));
    %
    
    %
    %Take Numbers from Guner, Rauh and Ventura for transfers
    
    alpha=-1.816;
    
    inc_norm=income./55167.55;
    inc_norm=income./ 75932.91;
    beta_0=-4.290  ;
    beta_1=-0.006;
    
    transfer_norm = exp(alpha).*exp(beta_0*inc_norm).*inc_norm.^beta_1;
    
    transfer_2012 = transfer_norm*75932.91;
    transfer_2012 = transfer_norm.*income;
    transfer_2012 = transfer_norm*75932.91;
    
    
    transfer=zeros(n,1);
    help=find(transfer_2012<0.00000001,1,'first');
    for i=1:help
        
        
        transfer(i)   = interp1(income,transfer_2012,income(i)*110.9351/82.59,'spline','extrap');
        
    end
    
    %USE CPI Deflater from 2012 to 2000
    
    phase_out=-(transfer(2:n)-transfer(1:n-1))./(income(2:n)-income(1:n-1));
    
    phase_out(n,1)=0;
    t_prime=t_prime+phase_out;
    Tax=Tax-transfer;
    
    
    
    t_prime_prime = (t_prime(2:n)-t_prime(1:n-1))./(y(2:n)-y(1:n-1));
    t_prime_prime(n,1)=t_prime_prime(n-1,1);
    
    
    
    
    %Left panel of Figure 5
    hold off
    p1=plot(income/1000,t_prime_hsv)
    set(p1,'Color','red','LineWidth',2.5);
    set(gca,'FontSize',18)
    hold on
    p3=plot(income/1000,(t_prime));
    set(p3,'Color','black','LineWidth',2.5,'LineStyle','--');
    xlim([0 500])
    ylim([-0.2 0.9])
    %ylabel(' Share of Effect');
    h= xlabel('Income in \$1,000');
    set(h,'Interpreter','latex')
    set(gca,'FontSize',18)
    ylabel('Initial Marginal Tax Rates');
    
    hleg1 = legend('CRP  ','Gouveia-Strauss + Transfer Phase-Out');
    
    set(hleg1,'Interpreter','latex')
    
    
    
end



%Calibrate the wage that is asociate with a given income level
wage_cal = ((income.^(1/elasticity))./(1-t_prime)).^(1/(1+(1/elasticity)));%wages given current policies


l_cal=income./wage_cal;%hours worked given current policies


%Given that we knwo GDP, wages and hours worked, we can easily get productivities
gdp_cal=sum(pmf.*income);

x_cal=gdp_cal^rho;

a = wage_cal./((pmf.*l_cal).^(rho-1).*x_cal.^(1/rho-1));


w=wage_cal;

dw=    [(w(2:n)-w(1:n-1));0];
dw(n)=dw(n-1);
dw_cal=dw;


transfer=0;

c=transfer+y-Tax;
inner_utility = c - l_cal.^(1+(1/elasticity))./(1+1/elasticity);

u_prime = inner_utility.^(-gamma_crra);
cum_u_prime=flipud(cumsum(flipud(weights.*u_prime)));
cum_u_prime=[cum_u_prime(2:n);0];

lambda =   sum(weights.*u_prime);



for i=1:n
    g_bar(i) = cum_u_prime(i)./sum(pmf(i:n));
    
end
%


%Get the cross-wage elaticities

l=l_cal;
w=wage_cal;
y=income;
gamma_bar=zeros(n,n);

%Get

for i=1:n
    
    gamma_bar(:,i) =ones(n,1)* (1-rho)*(a(i).*(pmf(i).*l(i)).^rho)./sum(a.*(pmf.*l).^rho);
    
end



%now obtain equlibrium labor elasticities

e_tilde=elasticity.*(1-t_prime-y.*t_prime_prime)./(1-t_prime+y.*elasticity.*t_prime_prime);

e_tilde_tau=elasticity.*(1-t_prime)./(1-t_prime+y.*elasticity.*t_prime_prime);

e_tilde_tau_ge=e_tilde_tau./(1+e_tilde./el_sub);

e_tilde_ge=e_tilde./(1+e_tilde./el_sub);



%Now we consider elementary tax reforms at each possible income level
%and cacluate the resulting labor supply changes

dtau=0.01;

for i=1:n
    vector=zeros(n,1);
    vector(i)=1;
    d_ln_l(:,i) = - e_tilde_tau_ge(i).*(dtau./(1-t_prime(i))).*vector - e_tilde_tau_ge(i).*(dtau./(1-t_prime(i)))  .* ( gamma_bar(:,i).*e_tilde_ge)...
        ./(1 - sum(e_tilde_ge.*gamma_bar(i,:)'));
    
    
end





%calculate the resulting wage changes

for j=1:n
    
    
    d_ln_w(:,j) = gamma_bar*d_ln_l(:,j) +(rho-1).*d_ln_l(:,j);
    
end




%Now we calculte the revenue and welfare effects for each of these
%elementary tax reforms

delta_y=    [(y(2:n)-y(1:n-1));0];


delta_y(n)=delta_y(n-1);



%first get the classical effects that one also gets for exogenous wages
for j=1:n
    mechanical(j) = delta_y(j)*dtau*(1-cdf(j));
    labor_supply(j) = y(j)*pmf(j)*e_tilde_tau(j)*t_prime(j)/(1-t_prime(j))*dtau;%it is defined as a tax loss. so it being negative means an increase in tax revenue
    mechanical_utility(j) = mechanical(j)*(1-g_bar(j)/lambda);
end


%Now get the effects on utilities that come from the changes in the
%wages
for j=1:n
    for i=1:n
        delta_u(i,j) = d_ln_w(i,j)*(1-t_prime(i))*y(i);
    end
    utility_effect_ge(j) = sum((1/lambda)*weights.*u_prime.*delta_u(:,j));
end



%Finally, get the effect on tax revenue due to general equilibrium
%effects

for j=1:n
    
    for i=1:n
        dy(i,j) = d_ln_l(i,j)*y(i) + d_ln_w(i,j)*y(i);
        
        if i==j
            
            dy(i,j)=   d_ln_l(i,j)*y(i) + d_ln_w(i,j)*y(i) + e_tilde_tau(i)*y(i)*dtau/(1-t_prime(j));
            
        end
        
        delta_tau(i,j)=t_prime(i)*dy(i,j);
        
    end
    
    ge_revenue(j) = - sum(pmf.*delta_tau(:,j));%it is defined as a tax loss. so it being negative means an increase in tax revenue
end







l_over_m_w=  labor_supply./(mechanical_utility);
ge_over_m_w=   (-utility_effect_ge+ ge_revenue)./( mechanical_utility);


l_over_m_r=  labor_supply./(mechanical );
ge_over_m_r=   ( ge_revenue)./( mechanical );







 


%Figure 2 of the paper and right panel of figure 5
hold off
p1=plot(income/1000,(1-l_over_m_r))
set(p1,'Color','red','LineWidth',2.5);
set(gca,'FontSize',18)
hold on
p3=plot(income/1000,(1-l_over_m_r-ge_over_m_r));
set(p3,'Color','black','LineWidth',2.5,'LineStyle','--');
xlim([0 300])
ylim([0.6 1])
ylabel('Revenue Effect');
h= xlabel('$y^*$ in \$1,000');
set(h,'Interpreter','latex')
set(gca,'FontSize',18)

hleg1 = legend('$\hat{\mathcal{R}}_{ex}(y^*)$', '$\hat{\mathcal{R}}_{ge}(y^*)$');

set(hleg1,'Interpreter','latex')



 
return



%Figure 6 of the paper:
hold off
p1=plot(income/1000,(1-l_over_m_w))
set(p1,'Color','red','LineWidth',2.5);
set(gca,'FontSize',18)
hold on
p3=plot(income/1000,(1-l_over_m_w-ge_over_m_w));
set(p3,'Color','black','LineWidth',2.5,'LineStyle','--');
xlim([0 300])
%ylim([0.5 1.1])
ylabel(' Welfare Effect in Terms of Public Funds');
h= xlabel('$y^*$ in \$1,000');
set(h,'Interpreter','latex')
set(gca,'FontSize',18)

hleg1 = legend('$\hat{\mathcal{W}}_{ex}(y^*) $','$\hat{\mathcal{W}}_{ge}(y^*) $');

set(hleg1,'Interpreter','latex')






