

clc
clear all
format long;
format compact;
warning off;

options=optimset('TolCon',1e-13,'TolFun',1e-13,'TolX',1e-13,'Display','off','MaxFunEvals',10000000,'MaxIter',2000);

%Define Income grid
income_start=2500;
income_end=5002500

n=(income_end-income_start)/250+1;
income=linspace(income_start,income_end,n)';
y=income;

%elasticitiy of substitution
el_sub=3.1;
rho=1-1./el_sub;

%labor supply elasticity
elasticity=.33;





%Calibrate the income distribution. Start with lognormal part
mu_inc=10;
sigma=.95;
pareto=2.5;
exp(mu_inc+sigma^2/2)



%Now add Pareto tail for incomes >150 000, where the thickness parameter
%decreasses linearly between 150 000 and 350 000

pareto_start=150000
index_pareto_start=find(income>pareto_start,1,'first');


pareto_constant=350000;
index_pareto_constant=find(income>pareto_constant,1,'first');

cutoff=index_pareto_constant;
pareto_paraeter_start=1/((1-logncdf(pareto_start,mu_inc,sigma))./(lognpdf(pareto_start,mu_inc,sigma).*pareto_start))


pareto_vector=linspace(pareto_paraeter_start-1.55,0,index_pareto_constant-index_pareto_start+1);
if pareto_paraeter_start-pareto_paraeter_start>1
    disp('uncorrect smoothing')
    return
end
vector_help=linspace(0,(pareto_paraeter_start-1.55),index_pareto_constant-index_pareto_start+1);

vector = 1./(1+exp(-1 .*(vector_help-(pareto_paraeter_start-1.5)./2)));
vector=vector';
vector2=flipud(vector_help');

pareto_vector=1.5+vector2;

plot(pareto_vector)


pmf=lognpdf(income,mu_inc,sigma);
interesting=zeros(n,1);
for i=index_pareto_start:index_pareto_constant
    if i>index_pareto_start
        pmf(i) = ( pareto*x_min^pareto*income(i)^(-pareto-1));
        
    end
    pareto=pareto_vector(i-index_pareto_start+1);
    x_min = ((pmf(i)*income(i)^(pareto+1))/pareto)^(1/pareto);
    
    pmf(i) = ( pareto*x_min^pareto*income(i)^(-pareto-1));
    interesting(i) = x_min;
    
end
for i=index_pareto_constant:n
    pmf(i) = ( pareto*x_min^pareto*income(i)^(-pareto-1));
end

pmf= ksdensity(income,income,'weights',pmf,'width',20000);

%probability mass function of income
pmf=pmf./sum(pmf);

%cdf of income
cdf=cumsum(pmf);





%%Calibrate current tax schedule.

tau_tax=-3;
p_tax=.15

tax_marg = 1 - (1-tau_tax) .* income.^(-p_tax);

tau=tax_marg;

%Calibrate wages that are associated with each income level as in
%Saez (2001)

wage_cal = ((income.^(1/elasticity))./(1-tau)).^(1/(1+(1/elasticity)));%wages given current policies


l_cal=income./wage_cal;%hours worked given current policies


%Given that we knwo GDP, wages and hours worked, we can easily get productivities
gdp_cal=sum(pmf.*income);

x_cal=gdp_cal^rho;

a = wage_cal./((pmf.*l_cal).^(rho-1).*x_cal.^(1/rho-1));


w=wage_cal;
dw_up=[(w(2:n)-w(1:n-1));0];
dw_up(n)=dw_up(n-1);
dw_down=[0;(w(2:n)-w(1:n-1))];
dw_down(1)=dw_down(2);


dw=    [(w(2:n)-w(1:n-1));0];
dw(n)=dw(n-1);
dw=.5*dw_up+.5*dw_down;
dw=dw_up;
dw_cal=dw;







%Consider tax revenue maximizing planer
weights=zeros(n,1);
cumweights=cumsum(weights);

%The associated marginal value of public funds is given by
lambda=1;




%get the tax schedule that would be optimal if the wage distribution were
%exogenous -- it is obtained in closed form

mu=lambda*(1-cdf);

t_prime_current = 1- ( 1 + (mu./(w.*pmf)).*dw * (1+1/elasticity)).^(-1);


for i=cutoff+1:n
    t_prime_current(i)=t_prime_current(cutoff);
end

%get associated labor supply nad income levels with these marginal
%tax rates (again erroneously assuming that wages are fixed)
l_pe_naive=(wage_cal.*(1-t_prime_current)).^elasticity;
y_pe_naive=wage_cal.*l_pe_naive;

adjust_wage=.05;

wage_dev=1;
loop_wage=0
w_pe(:)=wage_cal;
while ( wage_dev>0.0000001)
    loop_wage=loop_wage+1;
    wage_old=w_pe(:);
    l_pe(:)=(w_pe(:).*(1-t_prime_current)).^elasticity;
    
    wage_new = a.*(pmf.*l_pe(:)).^(rho-1)*(sum( a.*(pmf.*l_pe(:)).^rho ))^(1/rho-1);
    wage_new(1)
    wage_dev = max(abs ((wage_new-wage_old)./wage_old));
    
    w_pe(:)=adjust_wage*wage_new+(1-adjust_wage)*wage_old;
    
end

y_pe(:)=w_pe(:).*l_pe(:);




%set the tax constant for the income level above which the Pareto tail is
%constant (would be constant if the distribution was truly unbounded)


%Set some parameters for start in first loop.
for i=1:n
    tax(i,1) = taxes_(l_cal(i), i, wage_cal, income, tax_marg);
end


transfer=sum(pmf.*tax);

c=transfer+income-tax;
inner_utility = c - l_cal.^(1+(1/elasticity))./(1+1/elasticity);

w=wage_cal;
l=l_cal;
t_prime=ones(n,1)*.3;
y=income;



%Set "looop" parametsers
adjust1=.05;
adjust2=.05;
adjust_wage=.05;
loop=0
tax_dev=1;




while ( tax_dev>0.00001)
    loop=loop+1
    
    %Adjust dw vector because of new wage distribution
    
    dw_up=[(w(2:n)-w(1:n-1));0];
    dw_up(n)=dw_up(n-1);
    dw_down=[0;(w(2:n)-w(1:n-1))];
    dw_down(1)=dw_down(2);
    dw=.5*dw_down+.5*dw_up;
    dw=dw_up;
    
    
    
    %Simulate optimal marginal tax rate formula
    l_growth(1:n-1,1)=(l(2:n)-l(1:n-1))./l(1:n-1);
    l_growth(n)=l_growth(n-1);
    w_growth(1:n-1,1)=(w(2:n)-w(1:n-1))./w(1:n-1);
    w_growth(n)=w_growth(n-1);
    
    
    
    
    new_term=l_growth./w_growth;
    
    t_prime_prime = (t_prime(2:n)-t_prime(1:n-1))./(y(2:n)-y(1:n-1));
    t_prime_prime(n,1)=t_prime_prime(n-1,1);
    e_tilde=elasticity.*(1-t_prime-y.*t_prime_prime)./(1-t_prime+y.*elasticity.*t_prime_prime);
    
    RHS1=(1+1/elasticity)*(mu./(lambda*pmf.*w)).*dw;
    RHS2=(1+1/elasticity)*(mu./(lambda*pmf.*w)).*dw .*(new_term./el_sub);
    RHS3=((weights)./(lambda*pmf)-1).*(1/el_sub);
    RHS3=((weights-lambda*pmf)./(pmf.*lambda)).*(1/el_sub);
    RHS=RHS1+RHS2+RHS3;
    
    
    
    t_prime_new=RHS./(1+RHS);
    
    
    
    
    
    %slowly adjust the scheule of marginal tax rates
    
    t_prime_old=t_prime;
    
    converge=cutoff;
    
    %Since theory tells us that above 350k (the grid point that is associated with 350k in the initial tax equilibrium), the tax rate is constant for an
    %unbounded support, we focus on the rates up
    tax_dev = max(abs( (t_prime_new(1:converge)-t_prime(1:converge)) ))
    
    for i=1:converge
        t_prime(i,1)=adjust1.*t_prime_new(i,1) + (1-adjust1).*t_prime_old(i,1);
    end
    
    for i=converge+1:n
        t_prime(i,1)=adjust2.*t_prime_new(i,1) + (1-adjust2).*t_prime_old(i,1);
        
    end
    for i=cutoff+1:n
        
        t_prime(i)=t_prime(cutoff);
    end
    
    
    
    %Now get the new wage distribution given these marginal tax rates
    
    wage_old=w;
    loop_wage=0;
    wage_dev=1;
    
    while ( wage_dev>0.0000001)
        loop_wage=loop_wage+1;
        wage_old=w;
        l=(w.*(1-t_prime)).^elasticity;
        
        wage_new = a.*(pmf.*l).^(rho-1)*(sum( a.*(pmf.*l).^rho ))^(1/rho-1);
        
        wage_dev = max(abs ((wage_new-wage_old)./wage_old));
        
        w=adjust_wage*wage_new+(1-adjust_wage)*wage_old;
        
    end
    
    
    
    
    
    dw_up=[(w(2:n)-w(1:n-1));0];
    dw_up(n)=dw_up(n-1);
    dw_down=[0;(w(2:n)-w(1:n-1))];
    dw_down(1)=dw_down(2);
    dw=.5*dw_down+.5*dw_up;
    dw=dw_up;
    
    
    
    
    
    y=w.*l;
    
    
    for i=1:n
        tax(i,1) = taxes_(l(i), i, w, y, t_prime);
    end
    
    transfer=sum(pmf(1:n).*tax(1:n));
    
    
    
    
end




save optimum.mat



l_pe_naive=(wage_cal.*(1-t_prime_current)).^elasticity;
y_pe_naive=wage_cal.*l_pe_naive;


%Left panel of Figure 4.
hold off
p1=plot(income/1000,t_prime_current);
set(p1,'Color','black','LineWidth',2,'LineStyle',':');
hold on
p1=plot(income/1000,t_prime);
set(p1,'Color','red','LineWidth',2,'LineStyle','-');

xlim([0 500])
set(gca,'FontSize',18)

ylabel('Marginal Tax Rates');
xlabel('Type expressed by current income in $1,000');
hleg1 = legend('Exogenous-Wage Planner','Elasticity of Substitution 3.1' );

set(gca,'FontSize',18)

 